#!/usr/bin/env python3
import sys
from pathlib import Path


def install(pkg: str, *pipargs: str):
    sys.path += [] if (sp := f"{Path(__file__)}.site-packages") in sys.path else [sp]
    try:
        __import__(pkg)
    except ImportError:
        install_args = ["install", "--upgrade", f"--target={sp}", *(pipargs or [pkg])]
        getattr(__import__("pip"), "main")(install_args)
        __import__(pkg)


install("pyxbar", Path(__file__).parent.parent.as_posix())


from pyxbar import Config, Divider, Menu, MenuItem, get_config  # noqa: E402


class MyConfig(Config):
    MYVARIABLE: bool = True
    ...


CONFIG = get_config(MyConfig)  # type: ignore


if __name__ == "__main__":
    Menu("some title").with_items(
        Divider(),  # shortcut for ---, also handles submenu depth
        MenuItem(
            "👁️ overview",
            title_alternate="👁️ overview-alt",
        )
        .with_submenu(
            MenuItem("hi, i'm a submenu!").with_alternate(
                MenuItem("hi, i'm alternate!"),
            ),
            MenuItem("hi, i'm monospace!", monospace=True),
        )
        .with_alternate("👁️ overview-alt", font="monospace", color="red"),
    ).print()



from __future__ import annotations

import logging
import os
from pathlib import Path
import re
from typing import (
    Any,
    Callable,
    Generator,
    Iterable,
    TypeVar,
    Union,
)

from .types import Numeric, Renderable

logger = logging.getLogger()
logging.basicConfig(level=logging.DEBUG, format="=====> %(message)s")

T = TypeVar("T")


def get_in(
    keys: Union[str, list[str]],
    coll: Union[dict[Union[str, int], Any], list[Any]],
    default: object = (_no_default := object()),
) -> Generator[Any, None, None]:
    try:
        key, *keys = keys.split(".") if isinstance(keys, str) else keys
        if key == "*":
            for i in range(len(coll)):
                yield from get_in([i, *keys], coll, default)
        elif not keys:
            yield coll[key]
        else:
            yield from get_in(keys, coll[key], default)
    except (KeyError, IndexError, TypeError):
        if default is not _no_default:
            yield default


def strify(joiner: str, *args: Any) -> str:
    """joins strings and filters out None values"""
    if len(args) == 1 and not isinstance(args[0], str):
        return strify(joiner, *args[0])
    return joiner.join(str(arg) for arg in args if arg is not None)


def camel_to_snake(
    s: str, _cache: dict[str, Any] = {"ID": "id", "_re": re.compile(r"(?<!^)(?=[A-Z])")}
) -> str:
    if s in _cache:
        return _cache[s]
    return _cache.setdefault(s, _cache["_re"].sub("_", s).lower())


def with_something(
    ret: T,
    key: list[Renderable],
    *children: Union[Renderable, Iterable[Renderable]],
) -> T:
    for child in children:
        if isinstance(child, Iterable):
            key.extend(child)
        else:
            key.append(child)
    return ret


def threshold_icons(level: Numeric, *icons: tuple[str, int], default: str = "") -> str:
    """return icons based on where a number falls in a list"""
    icon = default
    for icon, limit in icons:
        if level >= limit:
            return icon
    else:
        return default or icon


def threshold_traffic_icons(
    level: Numeric,
    blue: int,
    green: int,
    yellow: int,
    red: int,
    zero: str = "",
):
    return threshold_icons(
        level,
        ("🔵", blue),
        ("🟢", green),
        ("🟡", yellow),
        ("🔴", red),
        default=zero,
    )

def cache_path(__file__: str, name: str, ensure: Union[Callable, None] = None) -> Path:
    path = (
        Path(os.environ.get("XDG_CACHE_HOME", "~/.cache"))
        / "pyxbar"
        / Path(__file__).name
        / name
    ).expanduser()

    if ensure:
        path.parent.mkdir(exist_ok=True, parents=True)
        ensure(path, exist_ok=True)

    return path
    
def cache_file(__file__: str, name: str, touch: bool = False) -> Path:
    return cache_path(__file__, name, Path.touch if touch else None)

def cache_dir(__file__: str, name: str, mkdir: bool = False) -> Path:
    return cache_path(__file__, name, Path.mkdir if mkdir else None)



from __future__ import annotations

import json
import shlex
import subprocess
from dataclasses import dataclass, field
from os.path import expanduser
from pathlib import Path
from shlex import quote
from typing import (
    Any,
    Callable,
    Generator,
    Iterable,
    TypeVar,
    Union,
)

from pyxbar.config import get_config

from .menu import MenuItem
from .types import Renderable, RenderableGenerator
from .utils import camel_to_snake, get_in


@dataclass
class Divider(MenuItem):
    """Shortcut for a divider, super handy"""

    title: str = "---"


@dataclass
class MonoItem(MenuItem):
    """Shortcut for a monospace text menu item"""

    color: str = "white"
    monospace: Union[bool, None] = True


@dataclass
class ShellItem(MenuItem):
    """Menu Item that runs a shell script when selected"""

    cwd: Union[str, Path, None] = None

    def __init__(
        self,
        title: str,
        shell: str,
        cwd: Union[str, Path, None] = None,
        **kwargs: Any,
    ):
        super().__init__(title=title, shell=shell, **kwargs)
        if cwd is not None:
            self.cwd = cwd

    def __post_init__(self):
        if isinstance(self.cwd, str):
            self.cwd = Path(self.cwd)

        if self.cwd and not self.cwd.exists():
            self.config.error(f"❌ cwd does not exist at {self.cwd}")

        if self.shell and not self.params:
            self.shell, *self.params = (quote(_) for _ in shlex.split(self.shell))

    def shell_params(self, use_cwd: bool = True) -> Iterable[str]:
        shell_params = super().shell_params()

        if use_cwd and self.cwd:
            shell_params = ("cd", expanduser(self.cwd), "&&", *shell_params)

        return shell_params

    def shell_str(self, use_cwd: bool = False) -> str:
        return " ".join(self.shell_params(use_cwd=use_cwd))

    def subclass_render_hook(self, depth: int = 0) -> Generator[Renderable, None, None]:
        if self.config.DEBUG:
            yield MenuItem(
                title=f"╰─ {self.shell_str(use_cwd=False)}",
                font=get_config().MONO_FONT,
                disabled=True,
            )

    def check_output(self, check: bool = True) -> str:
        shell_params = list(self.shell_params(use_cwd=False))
        if self.config.DEBUG:
            self.logger.debug(f"running: {shell_params}")
        try:
            output = subprocess.check_output(shell_params, cwd=self.cwd)
        except subprocess.CalledProcessError:
            if check:
                raise
            return ""
        else:
            return output.decode("utf-8").strip()


class DataclassItem:
    """
    Helpers to turn attributes in a dataclass to menu items
    """

    def menu_item_from_attr(
        self, name: str, attr: str, tr: Callable[[str], str] = str
    ) -> MenuItem:
        value = tr(getattr(self, attr))
        return MenuItem(name.format(value=value, self=self))

    def submenu_item_from_attr(
        self,
        name: str,
        attr: str,
        delim: str = "",
        tr: Callable[[str], str] = str,
        sort: Union[bool, Callable[[str], Any]] = False,
        **menu_kwargs: Any,
    ) -> MenuItem:
        """
        create a menuitem from a class attribute
        name: the menu item namte
        attr: the attr name
        delim: split the

        """
        value = getattr(self, attr)
        if isinstance(value, str):
            items = value.split(delim) if delim else [value]
        else:
            items = value
            if not isinstance(items, Iterable):
                items = [items]

        items = [clean for i in items if (clean := tr(i).strip())]

        if sort:
            items.sort(key=(sort if callable(sort) else None))

        return MenuItem(
            f"{name} [{len(items)}]",
            only_if=items,
            **menu_kwargs,
        ).with_submenu(MenuItem(_) for _ in items)


FJ = TypeVar("FJ", bound="JsonItem")


@dataclass
class JsonItem(DataclassItem):
    kwargs: dict[str, object] = field(default_factory=dict)

    @classmethod
    def from_json_list(cls: type[FJ], json_str: str, **attrs: Any) -> list[FJ]:
        json_str = json_str.strip()
        if not json_str:
            return []

        if json_str[0] == "[" and json_str[-1] == "]":
            loaded = json.loads(json_str)
        else:
            loaded = [json.loads(_) for _ in json_str.splitlines()]

        return [cls.from_dict(**attrs, **_) for _ in loaded]

    @classmethod
    def from_json_dict(
        cls: type[FJ],
        json_str: Union[str, None],
        subkey: str = "",
        key_as_name: bool = True,
        **attrs: Any,
    ) -> dict[str, FJ]:
        loaded = json.loads(json_str or "{}")
        if subkey:
            loaded = next(get_in(subkey, loaded))

        return {
            k: cls.from_dict(
                **attrs, **loaded[k], **({"name": k} if key_as_name else {})
            )
            for k in loaded
        }

    @classmethod
    def from_json(cls, json_str: str) -> JsonItem:
        loaded = json.loads(json_str)
        return cls.from_dict(**loaded)

    @classmethod
    def from_dict(cls, **kwargs: Any) -> JsonItem:
        return cls(
            **{
                camel_to_snake(k): kwargs[k]
                for k in kwargs
                if camel_to_snake(k) in cls.__dataclass_fields__
            },
            kwargs={k: kwargs[k] for k in kwargs if k not in cls.__dataclass_fields__},
        )

    def with_attrs(self: FJ, **kwargs: Any) -> FJ:
        for key, value in kwargs.items():
            setattr(self, key, value)
        return self



from .config import Config as Config
from .config import get_config as get_config
from .menu import Menu as Menu
from .menu import MenuItem as MenuItem
from .menu import Renderable as Renderable
from .menu_items import DataclassItem as DataclassItem
from .menu_items import Divider as Divider
from .menu_items import JsonItem as JsonItem
from .menu_items import MonoItem as MonoItem
from .menu_items import ShellItem as ShellItem
from .types import Renderable as Renderable
from .types import RenderableGenerator as RenderableGenerator
from .utils import camel_to_snake as camel_to_snake
from .utils import get_in as get_in
from .utils import strify as strify
from .utils import cache_file as cache_file
from .utils import cache_dir as cache_dir

__version__ = "0.3.6"

__all__ = ("__version__",)
